'use strict';

const MagicString = require('magic-string');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const MagicString__default = /*#__PURE__*/_interopDefaultLegacy(MagicString);

async function compileApplet(body, ctx, options = {}) {
  const {
    classPrefix = "uno-",
    hashFn = hash,
    layer = "applet_shortcuts"
  } = options;
  const { uno, tokens } = ctx;
  const replacements = [];
  const result = await Promise.all(body.split(/\s+/).filter(Boolean).map(async (i) => [i, !!await uno.parseToken(i)]));
  const known = result.filter(([, matched]) => matched).map(([i]) => i);
  const unknown = result.filter(([, matched]) => !matched).map(([i]) => i);
  replacements.push(...unknown);
  body = known.join(" ");
  if (body) {
    const hash2 = hashFn(body);
    const className = `${classPrefix}${hash2}`;
    replacements.unshift(className);
    uno.config.shortcuts.push([className, body, { layer }]);
    tokens.add(className);
  }
  return replacements;
}
function hash(str) {
  let i;
  let l;
  let hval = 2166136261;
  for (i = 0, l = str.length; i < l; i++) {
    hval ^= str.charCodeAt(i);
    hval += (hval << 1) + (hval << 4) + (hval << 7) + (hval << 8) + (hval << 24);
  }
  return `00000${(hval >>> 0).toString(36)}`.slice(-6);
}

const elementRE = /<\w(?=.*>)[\w:\.$-]*\s(((".*?>?.*?")|.*?)*?)\/?>/gs;
const valuedAttributeRE = /([?]|(?!\d|-{2}|-\d)[a-zA-Z0-9\u00A0-\uFFFF-_:!%-]+)(?:={?(["'])([^\2]*?)\2}?)?/g;
const charReg = /[.:%!#()[\/\],]/;
const string1RE = /([']).*?(['])/g;
const string2RE = /([\`]).*?([\`])/g;
function transformerApplet(options = {}) {
  return {
    name: "transformer-applet",
    enforce: "pre",
    async transform(s, _, ctx) {
      let code = new MagicString__default(s.toString());
      const elementMatches = code.original.matchAll(elementRE);
      for (const eleMatch of elementMatches) {
        const start = eleMatch.index;
        let matchStrTemp = eleMatch[0];
        const valuedAttributes = Array.from((eleMatch[1] || "").matchAll(valuedAttributeRE));
        for (const attribute of valuedAttributes) {
          const name = attribute[1];
          const content = attribute[3];
          if (!content)
            continue;
          if (["class", "className", "hover-class"].includes(name)) {
            if (name.includes(":"))
              continue;
            if (charReg.test(content)) {
              const replacements = await compileApplet(content, ctx, options);
              matchStrTemp = matchStrTemp.replace(content, replacements.join(" "));
            }
          }
        }
        if (eleMatch[0] !== matchStrTemp)
          code.overwrite(start, start + eleMatch[0].length, matchStrTemp);
      }
      code = new MagicString__default(code.toString());
      const string1Matches = [...code.original.matchAll(string1RE)];
      for (const match of string1Matches) {
        if (/\.(png|jpg|jpeg|gif|svg)/g.test(match[0]))
          continue;
        if (/http(s)?:\/\//g.test(match[0]))
          continue;
        const start = match.index;
        const body = match[0].slice(1, -1);
        if (charReg.test(body)) {
          const replacements = await compileApplet(body, ctx, options);
          code.overwrite(start, start + match[0].length, `'${replacements.join(" ")}'`);
        }
      }
      const string2Matches = [...code.original.matchAll(string2RE)];
      for (const match of string2Matches) {
        if (/\$\{.*\}/g.test(match[0]))
          continue;
        const start = match.index;
        const body = match[0].slice(1, -1);
        if (charReg.test(body)) {
          const replacements = await compileApplet(body, ctx, options);
          code.overwrite(start, start + match[0].length, `'${replacements.join(" ")}'`);
        }
      }
      s.overwrite(0, s.original.length, code.toString());
    }
  };
}

module.exports = transformerApplet;
